@@ -54,10 +54,13 @@ __all__ = [
     'FortranCompiler',
     'G95FortranCompiler',
     'GnuCCompiler',
+    'ElbrusCCompiler',
     'GnuCompiler',
     'GnuCPPCompiler',
+    'ElbrusCPPCompiler',
     'GnuDCompiler',
     'GnuFortranCompiler',
+    'ElbrusFortranCompiler',
     'GnuObjCCompiler',
     'GnuObjCPPCompiler',
     'IntelCompiler',
@@ -115,16 +118,20 @@ from .compilers import (
     IntelCompiler,
 )
 from .c import (
+    ArmCCompiler,
     CCompiler,
     ClangCCompiler,
     GnuCCompiler,
+    ElbrusCCompiler,
     IntelCCompiler,
     VisualStudioCCompiler,
 )
 from .cpp import (
+    ArmCPPCompiler,
     CPPCompiler,
     ClangCPPCompiler,
     GnuCPPCompiler,
+    ElbrusCPPCompiler,
     IntelCPPCompiler,
     VisualStudioCPPCompiler,
 )
@@ -139,6 +146,7 @@ from .fortran import (
     FortranCompiler,
     G95FortranCompiler,
     GnuFortranCompiler,
+    ElbrusFortranCompiler,
     IntelFortranCompiler,
     NAGFortranCompiler,
     Open64FortranCompiler,
