@@ -31,6 +31,7 @@ class DependenciesHelper:
         self.priv_libs = []
         self.priv_reqs = []
         self.cflags = []
+        self.version_reqs = {}
 
     def add_pub_libs(self, libs):
         libs, reqs, cflags = self._process_libs(libs, True)
@@ -44,30 +45,60 @@ class DependenciesHelper:
         self.priv_reqs += reqs
 
     def add_pub_reqs(self, reqs):
-        self.pub_reqs += mesonlib.stringlistify(reqs)
+        self.pub_reqs += self._process_reqs(reqs)
 
     def add_priv_reqs(self, reqs):
-        self.priv_reqs += mesonlib.stringlistify(reqs)
+        self.priv_reqs += self._process_reqs(reqs)
+
+    def _process_reqs(self, reqs):
+        '''Returns string names of requirements'''
+        processed_reqs = []
+        for obj in mesonlib.listify(reqs, unholder=True):
+            if hasattr(obj, 'generated_pc'):
+                processed_reqs.append(obj.generated_pc)
+            elif hasattr(obj, 'pcdep'):
+                pcdeps = mesonlib.listify(obj.pcdep)
+                for d in pcdeps:
+                    processed_reqs.append(d.name)
+                    self.add_version_reqs(d.name, obj.version_reqs)
+            elif isinstance(obj, dependencies.PkgConfigDependency):
+                if obj.found():
+                    processed_reqs.append(obj.name)
+                    self.add_version_reqs(obj.name, obj.version_reqs)
+            elif isinstance(obj, str):
+                name, version_req = self.split_version_req(obj)
+                processed_reqs.append(name)
+                self.add_version_reqs(name, version_req)
+            elif isinstance(obj, dependencies.Dependency) and not obj.found():
+                pass
+            else:
+                raise mesonlib.MesonException('requires argument not a string, '
+                                              'library with pkgconfig-generated file '
+                                              'or pkgconfig-dependency object, '
+                                              'got {!r}'.format(obj))
+        return processed_reqs
 
     def add_cflags(self, cflags):
         self.cflags += mesonlib.stringlistify(cflags)
 
     def _process_libs(self, libs, public):
-        libs = mesonlib.listify(libs)
+        libs = mesonlib.listify(libs, unholder=True)
         processed_libs = []
         processed_reqs = []
         processed_cflags = []
         for obj in libs:
-            if hasattr(obj, 'held_object'):
-                obj = obj.held_object
+            shared_library_only = getattr(obj, 'shared_library_only', False)
             if hasattr(obj, 'pcdep'):
                 pcdeps = mesonlib.listify(obj.pcdep)
-                processed_reqs += [i.name for i in pcdeps]
+                for d in pcdeps:
+                    processed_reqs.append(d.name)
+                    self.add_version_reqs(d.name, obj.version_reqs)
             elif hasattr(obj, 'generated_pc'):
                 processed_reqs.append(obj.generated_pc)
             elif isinstance(obj, dependencies.PkgConfigDependency):
                 if obj.found():
                     processed_reqs.append(obj.name)
+                    self.add_version_reqs(obj.name, obj.version_reqs)
             elif isinstance(obj, dependencies.ThreadDependency):
                 processed_libs += obj.get_compiler().thread_link_flags(obj.env)
                 processed_cflags += obj.get_compiler().thread_flags(obj.env)
@@ -75,25 +106,27 @@ class DependenciesHelper:
                 if obj.found():
                     processed_libs += obj.get_link_args()
                     processed_cflags += obj.get_compile_args()
-            elif isinstance(obj, build.SharedLibrary):
+            elif isinstance(obj, build.SharedLibrary) and shared_library_only:
+                # Do not pull dependencies for shared libraries because they are
+                # only required for static linking. Adding private requires has
+                # the side effect of exposing their cflags, which is the
+                # intended behaviour of pkg-config but force Debian to add more
+                # than needed build deps.
+                # See https://bugs.freedesktop.org/show_bug.cgi?id=105572
                 processed_libs.append(obj)
                 if public:
                     if not hasattr(obj, 'generated_pc'):
                         obj.generated_pc = self.name
-            elif isinstance(obj, build.StaticLibrary):
-                # Due to a "feature" in pkgconfig, it leaks out private dependencies.
-                # Thus we will not add them to the pc file unless the target
-                # we are processing is a static library.
-                #
-                # This way (hopefully) "pkgconfig --libs --static foobar" works
-                # and "pkgconfig --cflags/--libs foobar" does not have any trace
-                # of dependencies that the build file creator has not explicitly
-                # added to the dependency list.
+            elif isinstance(obj, (build.SharedLibrary, build.StaticLibrary)):
                 processed_libs.append(obj)
                 if public:
                     if not hasattr(obj, 'generated_pc'):
                         obj.generated_pc = self.name
-                    self.add_priv_libs(obj.get_dependencies())
+                if isinstance(obj, build.StaticLibrary) and public:
+                    self.add_pub_libs(obj.get_dependencies(internal=False))
+                    self.add_pub_libs(obj.get_external_deps())
+                else:
+                    self.add_priv_libs(obj.get_dependencies(internal=False))
                     self.add_priv_libs(obj.get_external_deps())
             elif isinstance(obj, str):
                 processed_libs.append(obj)
@@ -102,14 +135,55 @@ class DependenciesHelper:
 
         return processed_libs, processed_reqs, processed_cflags
 
-    def remove_dups(self):
-        self.pub_libs = list(set(self.pub_libs))
-        self.pub_reqs = list(set(self.pub_reqs))
-        self.priv_libs = list(set(self.priv_libs))
-        self.priv_reqs = list(set(self.priv_reqs))
-        self.cflags = list(set(self.cflags))
+    def add_version_reqs(self, name, version_reqs):
+        if version_reqs:
+            if name not in self.version_reqs:
+                self.version_reqs[name] = set()
+            # Note that pkg-config is picky about whitespace.
+            # 'foo > 1.2' is ok but 'foo>1.2' is not.
+            # foo, bar' is ok, but 'foo,bar' is not.
+            new_vreqs = [s for s in mesonlib.stringlistify(version_reqs)]
+            self.version_reqs[name].update(new_vreqs)
 
-        # Remove from pivate libs/reqs if they are in public already
+    def split_version_req(self, s):
+        for op in ['>=', '<=', '!=', '==', '=', '>', '<']:
+            pos = s.find(op)
+            if pos > 0:
+                return s[0:pos].strip(), s[pos:].strip()
+        return s, None
+
+    def format_vreq(self, vreq):
+        # vreq are '>=1.0' and pkgconfig wants '>= 1.0'
+        for op in ['>=', '<=', '!=', '==', '=', '>', '<']:
+            if vreq.startswith(op):
+                return op + ' ' + vreq[len(op):]
+        return vreq
+
+    def format_reqs(self, reqs):
+        result = []
+        for name in reqs:
+            vreqs = self.version_reqs.get(name, None)
+            if vreqs:
+                result += [name + ' ' + self.format_vreq(vreq) for vreq in vreqs]
+            else:
+                result += [name]
+        return ', '.join(result)
+
+    def remove_dups(self):
+        def _fn(xs):
+            # Remove duplicates whilst preserving original order
+            result = []
+            for x in xs:
+                if x not in result:
+                    result.append(x)
+            return result
+        self.pub_libs = _fn(self.pub_libs)
+        self.pub_reqs = _fn(self.pub_reqs)
+        self.priv_libs = _fn(self.priv_libs)
+        self.priv_reqs = _fn(self.priv_reqs)
+        self.cflags = _fn(self.cflags)
+
+        # Remove from private libs/reqs if they are in public already
         self.priv_libs = [i for i in self.priv_libs if i not in self.pub_libs]
         self.priv_reqs = [i for i in self.priv_reqs if i not in self.pub_reqs]
 
@@ -179,11 +253,12 @@ class PkgConfigModule(ExtensionModule):
             if len(url) > 0:
                 ofile.write('URL: %s\n' % url)
             ofile.write('Version: %s\n' % version)
-            if len(deps.pub_reqs) > 0:
-                ofile.write('Requires: {}\n'.format(' '.join(deps.pub_reqs)))
-            if len(deps.priv_reqs) > 0:
-                ofile.write(
-                    'Requires.private: {}\n'.format(' '.join(deps.priv_reqs)))
+            reqs_str = deps.format_reqs(deps.pub_reqs)
+            if len(reqs_str) > 0:
+                ofile.write('Requires: {}\n'.format(reqs_str))
+            reqs_str = deps.format_reqs(deps.priv_reqs)
+            if len(reqs_str) > 0:
+                ofile.write('Requires.private: {}\n'.format(reqs_str))
             if len(conflicts) > 0:
                 ofile.write('Conflicts: {}\n'.format(' '.join(conflicts)))
 
@@ -233,20 +308,34 @@ class PkgConfigModule(ExtensionModule):
                       'subdirs', 'requires', 'requires_private', 'libraries_private',
                       'install_dir', 'extra_cflags', 'variables', 'url', 'd_module_versions'})
     def generate(self, state, args, kwargs):
-        if len(args) > 0:
-            raise mesonlib.MesonException('Pkgconfig_gen takes no positional arguments.')
+        default_version = state.project_version['version']
+        default_install_dir = None
+        default_description = None
+        default_name = None
+        mainlib = None
+        if len(args) == 1:
+            mainlib = getattr(args[0], 'held_object', args[0])
+            if not isinstance(mainlib, (build.StaticLibrary, build.SharedLibrary)):
+                raise mesonlib.MesonException('Pkgconfig_gen first positional argument must be a library object')
+            default_name = mainlib.name
+            default_description = state.project_name + ': ' + mainlib.name
+            install_dir = mainlib.get_custom_install_dir()[0]
+            if isinstance(install_dir, str):
+                default_install_dir = os.path.join(install_dir, 'pkgconfig')
+        elif len(args) > 1:
+            raise mesonlib.MesonException('Too many positional arguments passed to Pkgconfig_gen.')
 
         subdirs = mesonlib.stringlistify(kwargs.get('subdirs', ['.']))
-        version = kwargs.get('version', None)
+        version = kwargs.get('version', default_version)
         if not isinstance(version, str):
             raise mesonlib.MesonException('Version must be specified.')
-        name = kwargs.get('name', None)
+        name = kwargs.get('name', default_name)
         if not isinstance(name, str):
             raise mesonlib.MesonException('Name not specified.')
         filebase = kwargs.get('filebase', name)
         if not isinstance(filebase, str):
             raise mesonlib.MesonException('Filebase must be a string.')
-        description = kwargs.get('description', None)
+        description = kwargs.get('description', default_description)
         if not isinstance(description, str):
             raise mesonlib.MesonException('Description is not a string.')
         url = kwargs.get('url', '')
@@ -255,6 +344,8 @@ class PkgConfigModule(ExtensionModule):
         conflicts = mesonlib.stringlistify(kwargs.get('conflicts', []))
 
         deps = DependenciesHelper(filebase)
+        if mainlib:
+            deps.add_pub_libs(mainlib)
         deps.add_pub_libs(kwargs.get('libraries', []))
         deps.add_priv_libs(kwargs.get('libraries_private', []))
         deps.add_pub_reqs(kwargs.get('requires', []))
@@ -265,7 +356,7 @@ class PkgConfigModule(ExtensionModule):
         if dversions:
             compiler = state.environment.coredata.compilers.get('d')
             if compiler:
-                deps.add_cflags(compiler.get_feature_args({'versions': dversions}))
+                deps.add_cflags(compiler.get_feature_args({'versions': dversions}, None))
 
         def parse_variable_list(stringlist):
             reserved = ['prefix', 'libdir', 'includedir']
@@ -294,7 +385,7 @@ class PkgConfigModule(ExtensionModule):
         variables = parse_variable_list(mesonlib.stringlistify(kwargs.get('variables', [])))
 
         pcfile = filebase + '.pc'
-        pkgroot = kwargs.get('install_dir', None)
+        pkgroot = kwargs.get('install_dir', default_install_dir)
         if pkgroot is None:
             pkgroot = os.path.join(state.environment.coredata.get_builtin_option('libdir'), 'pkgconfig')
         if not isinstance(pkgroot, str):
@@ -304,5 +395,5 @@ class PkgConfigModule(ExtensionModule):
         res = build.Data(mesonlib.File(True, state.environment.get_scratch_dir(), pcfile), pkgroot)
         return ModuleReturnValue(res, [res])
 
-def initialize():
-    return PkgConfigModule()
+def initialize(*args, **kwargs):
+    return PkgConfigModule(*args, **kwargs)
