@@ -14,6 +14,7 @@
 
 
 import sys, struct
+import shutil, subprocess
 
 SHT_STRTAB = 3
 DT_NEEDED = 1
@@ -103,7 +104,7 @@ class SectionHeader(DataSizes):
             self.sh_addralign = struct.unpack(self.XWord, ifile.read(self.XWordSize))[0]
         else:
             self.sh_addralign = struct.unpack(self.Word, ifile.read(self.WordSize))[0]
-#Elf64_Xword
+# Elf64_Xword
         if is_64:
             self.sh_entsize = struct.unpack(self.XWord, ifile.read(self.XWordSize))[0]
         else:
@@ -127,8 +128,13 @@ class Elf(DataSizes):
     def __enter__(self):
         return self
 
+    def __del__(self):
+        if self.bf:
+            self.bf.close()
+
     def __exit__(self, exc_type, exc_value, traceback):
         self.bf.close()
+        self.bf = None
 
     def detect_elf_type(self):
         data = self.bf.read(6)
@@ -303,9 +309,9 @@ class Elf(DataSizes):
         # completely unrelated. This has already happened once with X.org.
         # Thus we want to keep this change as small as possible to minimize
         # the chance of obliterating other strings. It might still happen
-        # but our behaviour is identical to what chrpath does and it has
+        # but our behavior is identical to what chrpath does and it has
         # been in use for ages so based on that this should be rare.
-        if len(new_rpath) == 0:
+        if not new_rpath:
             self.remove_rpath_entry(entrynum)
         else:
             self.bf.seek(rp_off)
@@ -332,20 +338,74 @@ class Elf(DataSizes):
             entry.write(self.bf)
         return None
 
+def fix_elf(fname, new_rpath, verbose=True):
+    with Elf(fname, verbose) as e:
+        if new_rpath is None:
+            e.print_rpath()
+            e.print_runpath()
+        else:
+            e.fix_rpath(new_rpath)
+
+def get_darwin_rpaths_to_remove(fname):
+    out = subprocess.check_output(['otool', '-l', fname],
+                                  universal_newlines=True,
+                                  stderr=subprocess.DEVNULL)
+    result = []
+    current_cmd = 'FOOBAR'
+    for line in out.split('\n'):
+        line = line.strip()
+        if ' ' not in line:
+            continue
+        key, value = line.strip().split(' ', 1)
+        if key == 'cmd':
+            current_cmd = value
+        if key == 'path' and current_cmd == 'LC_RPATH':
+            rp = value.split('(', 1)[0].strip()
+            result.append(rp)
+    return result
+
+def fix_darwin(fname, new_rpath):
+    try:
+        rpaths = get_darwin_rpaths_to_remove(fname)
+    except subprocess.CalledProcessError:
+        # Otool failed, which happens when invoked on a
+        # non-executable target. Just return.
+        return
+    try:
+        for rp in rpaths:
+            subprocess.check_call(['install_name_tool', '-delete_rpath', rp, fname],
+                                  stdout=subprocess.DEVNULL,
+                                  stderr=subprocess.DEVNULL)
+        if new_rpath:
+            subprocess.check_call(['install_name_tool', '-add_rpath', new_rpath, fname],
+                                  stdout=subprocess.DEVNULL,
+                                  stderr=subprocess.DEVNULL)
+    except Exception as e:
+        raise
+        sys.exit(0)
+
+def fix_rpath(fname, new_rpath, verbose=True):
+    try:
+        fix_elf(fname, new_rpath, verbose)
+        return 0
+    except SystemExit as e:
+        if isinstance(e.code, int) and e.code == 0:
+            pass
+        else:
+            raise
+    if shutil.which('install_name_tool'):
+        fix_darwin(fname, new_rpath)
+    return 0
+
 def run(args):
     if len(args) < 1 or len(args) > 2:
         print('This application resets target rpath.')
         print('Don\'t run this unless you know what you are doing.')
         print('%s: <binary file> <prefix>' % sys.argv[0])
-        exit(1)
-    with Elf(args[0]) as e:
-        if len(args) == 1:
-            e.print_rpath()
-            e.print_runpath()
-        else:
-            new_rpath = args[1]
-            e.fix_rpath(new_rpath)
-    return 0
+        sys.exit(1)
+    fname = args[0]
+    new_rpath = None if len(args) == 1 else args[1]
+    return fix_rpath(fname, new_rpath)
 
 if __name__ == '__main__':
-    run(sys.argv[1:])
+    sys.exit(run(sys.argv[1:]))
